@@ -105,12 +105,12 @@ class FrameworkPipeline(Transformer):
                         f"({i} of {count_of_transformers}) ----"
                     )
                     self.progress_logger.start_mlflow_run(
-                        run_name=str(transformer), is_nested=True
+                        run_name=str(stage_name), is_nested=True
                     )
 
                     with ProgressLogMetric(
                         progress_logger=self.progress_logger,
-                        name=str(transformer) or "unknown",
+                        name=str(stage_name) or "unknown",
                     ):
                         self.progress_logger.log_event(
                             pipeline_name,
